@@ -12,7 +12,6 @@ from sphinx.domains import ObjType
 from sphinx.domains.std import GenericObject, Target
 from sphinx.errors import ExtensionError, SphinxError, VersionRequirementError
 from sphinx.extension import Extension
-from sphinx.io import create_publisher
 from sphinx.locale import __
 from sphinx.parsers import Parser as SphinxParser
 from sphinx.roles import XRefRole
@@ -26,7 +25,6 @@ if TYPE_CHECKING:
     from typing import Any, TypeAlias
 
     from docutils import nodes
-    from docutils.core import Publisher
     from docutils.nodes import Element, Node, TextElement
     from docutils.parsers import Parser
     from docutils.parsers.rst import Directive
@@ -38,7 +36,7 @@ if TYPE_CHECKING:
     from sphinx.config import Config
     from sphinx.domains import Domain, Index
     from sphinx.environment import BuildEnvironment
-    from sphinx.ext.autodoc import Documenter
+    from sphinx.ext.autodoc._legacy_class_based._documenters import Documenter
     from sphinx.util.docfields import Field
     from sphinx.util.typing import (
         ExtensionMetadata,
@@ -153,9 +151,6 @@ class SphinxComponentRegistry:
         #: additional transforms; list of transforms
         self.transforms: list[type[Transform]] = []
 
-        # private cache of Docutils Publishers (file type -> publisher object)
-        self.publishers: dict[str, Publisher] = {}
-
     @property
     def autodoc_attrgettrs(self) -> dict[type, Callable[[Any, str, Any], Any]]:
         return self.autodoc_attrgetters
@@ -375,11 +370,14 @@ class SphinxComponentRegistry:
     def get_source_parsers(self) -> dict[str, type[Parser]]:
         return self.source_parsers
 
-    def create_source_parser(self, app: Sphinx, filename: str) -> Parser:
+    def create_source_parser(
+        self, filename: str, *, config: Config, env: BuildEnvironment
+    ) -> Parser:
         parser_class = self.get_source_parser(filename)
         parser = parser_class()
         if isinstance(parser, SphinxParser):
-            parser.set_application(app)
+            parser._config = config
+            parser._env = env
         return parser
 
     def add_translator(
@@ -410,7 +408,9 @@ class SphinxComponentRegistry:
                     % (builder_name, handlers),
                 ) from exc
 
-    def get_translator_class(self, builder: Builder) -> type[nodes.NodeVisitor]:
+    def get_translator_class(
+        self, builder: type[Builder] | Builder
+    ) -> type[nodes.NodeVisitor]:
         try:
             return self.translators[builder.name]
         except KeyError:
@@ -420,7 +420,9 @@ class SphinxComponentRegistry:
                 msg = f'translator not found for {builder.name}'
                 raise AttributeError(msg) from err
 
-    def create_translator(self, builder: Builder, *args: Any) -> nodes.NodeVisitor:
+    def create_translator(
+        self, builder: type[Builder] | Builder, *args: Any
+    ) -> nodes.NodeVisitor:
         translator_class = self.get_translator_class(builder)
         translator = translator_class(*args)
 
@@ -589,15 +591,6 @@ class SphinxComponentRegistry:
 
         return _get_env_version(app.extensions)
 
-    def get_publisher(self, app: Sphinx, filetype: str) -> Publisher:
-        try:
-            return self.publishers[filetype]
-        except KeyError:
-            pass
-        publisher = create_publisher(app, filetype)
-        self.publishers[filetype] = publisher
-        return publisher
-
 
 def merge_source_suffix(app: Sphinx, config: Config) -> None:
     """Merge any user-specified source_suffix with any added by extensions."""
